open Gny;;

let count_initial_char =
  let rec count_initial_char_r c s n l =
    if (n<l) & (String.get s n)=c
    then count_initial_char_r c s (succ n) l
    else n
  in fun c s -> count_initial_char_r c s 0 (String.length s);;

let id_to_latex s =
  let l=String.length s in
  if l=0
  then "\bullet"
  else let c=succ (count_initial_char '\'' (String.sub s 1 (pred l))) in
  "{" ^ (String.sub s 0 c) ^ "}" ^
  (
  if c<l
  then "_{" ^ (String.sub s c (l-c)) ^ "}"
  else ""
      );;

let principal_to_latex = id_to_latex
and variable_to_latex = id_to_latex
and pkey_id_to_latex = id_to_latex
and unary_to_latex = id_to_latex
and binary_to_latex = id_to_latex;;

let rec gny_to_latex = function
    Is_told(p, x) ->
      Printf.sprintf "\\istold{%s}{%s}"
	(principal_to_latex p) (gny_to_latex x)
  | Possesses(p, x) ->
      Printf.sprintf "\\possesses{%s}{%s}"
	(principal_to_latex p) (gny_to_latex x)
  | Believes(p, x) ->
      Printf.sprintf "\\believes{%s}{%s}"
	(principal_to_latex p) (gny_to_latex x)
  | Once_conveyed(p, x) ->
      Printf.sprintf "\\onceconveyed{%s}{%s}"
	(principal_to_latex p) (gny_to_latex x)
  | Fresh(x) ->
      Printf.sprintf "\\fresh{%s}" (gny_to_latex x)
  | Recognizable(x) ->
      Printf.sprintf "\\recognizable{%s}" (gny_to_latex x)
  | Is_secret(p, q, x) ->
      Printf.sprintf "\\sharedsecret{%s}{%s}{%s}"
	(principal_to_latex p)
	(principal_to_latex q)
	(gny_to_latex x)
  | Is_public_key(p, k) ->
      Printf.sprintf "\\publickey{%s}{%s}"
	(principal_to_latex p)
	(pkey_id_to_latex k)
  | Encrypt_pub(x, k) ->
      Printf.sprintf "\\encrypt{%s}{\\pluskey{%s}}"
	(gny_to_latex x)
	(pkey_id_to_latex k)
  | Encrypt_priv(x, k) ->
      Printf.sprintf "\\encrypt{%s}{\\minuskey{%s}}"
	(gny_to_latex x)
	(pkey_id_to_latex k)
  | Encrypt_s(x, k) ->
      Printf.sprintf "\\encrypt{%s}{%s}"
	(gny_to_latex x)
	(gny_to_latex k)
  | Decrypt_s(x, k) ->
      Printf.sprintf "\\decrypt{%s}{%s}"
	(gny_to_latex x)
	(gny_to_latex k)
  | Unary_func(x, a) ->
      Printf.sprintf "\\applyi{%s}{%s}"
	(unary_to_latex x)
	(gny_to_latex a)
  | Binary_func(x, a, b) ->
      Printf.sprintf "\\applyii{%s}{%s}{%s}"
	(unary_to_latex x)
	(gny_to_latex a)
	(gny_to_latex b)
  | Variable(x) -> variable_to_latex x
  | Public_key(k) ->
      Printf.sprintf "\\pluskey{%s}" (pkey_id_to_latex k)
  | Private_key(k) ->
      Printf.sprintf "\\minuskey{%s}" (pkey_id_to_latex k)
  | Pair(_,_) as pair ->
      Printf.sprintf "\\left(%s\\right)" (pair_to_latex pair)
  | Not_originated_here(x) ->
      Printf.sprintf "\\noh{%s}" (gny_to_latex x)
  | Honest_and_competent(p) ->
      Printf.sprintf "\\honestandcompetent{%s}" (principal_to_latex p)
  | Jurisdiction(p, x) ->
      Printf.sprintf "\\jurisdiction{%s}{%s}"
	(principal_to_latex p) (gny_to_latex x)
  | Leading_to(x, y) ->
      Printf.sprintf "\\leadingto{%s}{%s}"
	(gny_to_latex x)
	(gny_to_latex y)

and pair_to_latex = function
    Pair(x, y) ->
      Printf.sprintf "%s, %s"
      	(gny_to_latex x)
	(pair_to_latex y)
  | x -> gny_to_latex x;;

let rec proof_to_latex prepend = function
    ((true, _), _) -> ""
  | ((false, formula), (Hypothesis label)) ->
      Printf.sprintf "%s{%s \\mbox{~\\textbf{%s}}}\n"
     	prepend (gny_to_latex formula) label
  | ((false, formula), (Rule(tag, hypotheses))) ->
      Printf.sprintf "%s\\infer[\\mbox{\\textsf{%s}}]{%s}{\n%s%s}\n" 
        prepend tag (gny_to_latex formula)
	(String.concat (prepend ^ "&\n")
	   (List.map (proof_to_latex (prepend ^ "  ")) hypotheses))
        prepend;;

let rec proof_to_latex2 =
  let spacing x = 0.75 *. (float x) in
  function space -> function
    ((true, _), _) -> ""
  | ((false, formula), (Hypothesis label)) ->
      Printf.sprintf
	",\\\\\n\\hspace*{%fem} from hypothesis \\textbf{%s}.\\\\\n"
	(spacing (space + 2)) label
  | ((false, formula), (Rule(tag, hypotheses))) ->
      ( Printf.sprintf
	 ",\\\\\n\\hspace*{%fem} applying rule \\textsf{%s} to:\\\\\n"
	 (spacing (space + 2)) tag)
      ^ (String.concat ""
	   (List.map (function
	       ((false, formula), _) as z ->
		 (Printf.sprintf "\\hspace*{%fem}$%s$"
		    (spacing space) (gny_to_latex formula))
		   ^ (proof_to_latex2 (space + 1) z)
	     | _ -> "") hypotheses));;
